@@ -96,10 +96,27 @@ kuryr_k8s_opts = [
     cfg.BoolOpt("test_services_without_selector", default=False,
                 help="Whether or not service without selector tests will be "
                 "running"),
+    cfg.BoolOpt("test_endpoints_object_removal", default=True,
+                help="Whether to check that LB members are deleted upon "
+                "endpoints object removal or not"),
     cfg.BoolOpt("test_configurable_listener_timeouts", default=False,
                 help="Whether or not listener timeout values are "
                 "configurable"),
     cfg.IntOpt("lb_members_change_timeout", default=1200,
                help="The max time (in seconds) it should take to adjust the "
                " number LB members"),
+    cfg.BoolOpt("enable_reconciliation", default=False,
+                help="Whether or not reconciliation is enabled"),
+    cfg.BoolOpt("enable_listener_reconciliation", default=False,
+                help="Whether or not listener reconciliation is enabled"),
+    cfg.IntOpt("lb_reconcile_timeout", default=600,
+               help="The max time (in seconds) it should take for LB "
+               "reconciliation. It doesn't include the LB build time."),
+    cfg.BoolOpt("trigger_namespace_upon_pod", default=False,
+                help="Whether or not Namespace should be handled upon Pod "
+                "creation"),
+    cfg.BoolOpt("annotation_project_driver", default=False,
+                help="Whether or not annotation project tests will be "
+                     "running"),
+
 ]
