@@ -48,16 +48,13 @@ KURYR_PORT_CRD_PLURAL = 'kuryrports'
 KURYR_LOAD_BALANCER_CRD_PLURAL = 'kuryrloadbalancers'
 K8S_ANNOTATION_PREFIX = 'openstack.org/kuryr'
 K8S_ANNOTATION_LBAAS_STATE = K8S_ANNOTATION_PREFIX + '-lbaas-state'
-K8S_ANNOTATION_LBAAS_RT_STATE = K8S_ANNOTATION_PREFIX + '-lbaas-route-state'
-KURYR_ROUTE_DEL_VERIFY_TIMEOUT = 30
 KURYR_CONTROLLER = 'kuryr-controller'
-OS_ROUTES_API = 'route.openshift.io'
-OS_ROUTES_VERSION = 'v1'
-OS_ROUTES_PLURAL = "routes"
 
 
 class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
 
+    credentials = ['admin', 'primary', ['lb_admin', 'load-balancer_admin']]
+
     @classmethod
     def skip_checks(cls):
         super(BaseKuryrScenarioTest, cls).skip_checks()
@@ -68,6 +65,7 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
     def setup_clients(cls):
         super(BaseKuryrScenarioTest, cls).setup_clients()
         cls.k8s_client = k8s_client
+        cls.lbaas = cls.os_roles_lb_admin.load_balancer_v2.LoadbalancerClient()
 
     @classmethod
     def resource_setup(cls):
@@ -194,6 +192,15 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
 
         return name, pod
 
+    @classmethod
+    def delete_endpoint(cls, ep_name, body=None, namespace="default"):
+        if body is None:
+            body = {}
+        cls.k8s_client.CoreV1Api().delete_namespaced_endpoints(
+            name=ep_name,
+            body=body,
+            namespace=namespace)
+
     @classmethod
     def delete_network_policy(cls, name, namespace='default'):
         body = cls.k8s_client.V1DeleteOptions()
@@ -206,21 +213,36 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
     def delete_pod(cls, pod_name, body=None, namespace="default"):
         if body is None:
             body = {}
-        cls.k8s_client.CoreV1Api().delete_namespaced_pod(
-            name=pod_name,
-            body=body,
-            namespace=namespace)
-        # TODO(apuimedo) This sleep to be replaced with a polling with
-        # timeout for the pod object to be gone from k8s api.
-        time.sleep(30)
+        try:
+            cls.k8s_client.CoreV1Api().delete_namespaced_pod(
+                name=pod_name,
+                body=body,
+                namespace=namespace)
+        except kubernetes.client.rest.ApiException as e:
+            if e.status != 404:
+                raise
+            LOG.debug("Pod %s was not found.", pod_name)
+        retries = 6
+        while retries > 0:
+            try:
+                cls.k8s_client.CoreV1Api().read_namespaced_pod(
+                    pod_name,
+                    namespace)
+                time.sleep(5)
+            except kubernetes.client.rest.ApiException as e:
+                if e.status != 404:
+                    LOG.warning("An exception occured: %s", e)
+                break
+        else:
+            LOG.debug("Timeout - Pod %s has not been deleted yet.", pod_name)
 
     @classmethod
     def wait_for_pod_status(cls, pod_name, namespace="default",
-                            pod_status=None, retries=30):
+                            pod_status=None, retries=6):
         while pod_status != cls.get_pod_status(
                 pod_name,
                 namespace=namespace):
-            time.sleep(1)
+            time.sleep(5)
             retries -= 1
             if retries == 0:
                 raise lib_exc.TimeoutException()
@@ -259,23 +281,40 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             pod = cls.k8s_client.CoreV1Api().read_namespaced_pod(pod_name,
                                                                  namespace)
         except kubernetes.client.rest.ApiException:
-            return None
+            return False
 
+        return cls.get_readiness_state_from_pod(pod,
+                                                container_name=container_name)
+
+    @classmethod
+    def get_readiness_state_from_pod(cls, pod, container_name=None):
         if container_name:
             for container in pod.status.container_statuses:
                 if container.name == container_name:
                     return container.ready
-        else:
+        elif pod.status.conditions:
             for condition in pod.status.conditions:
                 if condition.type == 'Ready':
-                    return condition.status
+                    return condition.status == 'True'
+        return False
 
     @classmethod
-    def get_pod_readiness(cls, pod_name, namespace="default",
-                          container_name=None):
+    def check_pods_ready_num(cls, namespace="default", label="", num_pods=1):
+        pods = cls.get_pod_list(namespace=namespace, label_selector=label)
+        ready_pods = sum([cls.get_readiness_state_from_pod(p) for p in pods])
+        return num_pods == ready_pods
+
+    @classmethod
+    def check_pods_status_num(cls, namespace="default", label="", num_pods=1,
+                              status="Running"):
+        pods = cls.get_pod_list(namespace=namespace, label_selector=label)
+        status_pods = sum([p.status.phase == status for p in pods])
+        return num_pods == status_pods
+
+    @classmethod
+    def get_pod_readiness(cls, pod_name, namespace="default"):
         LOG.info("Checking if pod {} is ready".format(pod_name))
-        return cls.get_readiness_state(pod_name, namespace=namespace,
-                                       container_name=container_name)
+        return cls.get_readiness_state(pod_name, namespace=namespace)
 
     @classmethod
     def get_container_readiness(cls, pod_name, namespace="default",
@@ -334,7 +373,7 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
 
     def exec_command_in_pod(self, pod_name, command, namespace="default",
                             stderr=False, container=None,
-                            req_timeout=60, f_timeout=2):
+                            req_timeout=60, f_timeout=10):
         api = self.k8s_client.CoreV1Api()
         kwargs = dict(command=command, stdin=False, stdout=True, tty=False,
                       stderr=stderr)
@@ -386,18 +425,25 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
                 return project['id']
 
     def create_deployment(self, deployment_name=None, api_version="apps/v1",
-                          kind="Deployment", namespace="default"):
+                          kind="Deployment", namespace="default",
+                          labels={"app": "demo"}, failing_probe=False):
         api_instance = kubernetes.client.AppsV1Api()
         if not deployment_name:
             deployment_name = data_utils.rand_name(prefix='kuryr-deployment')
         deployment = k8s_client.V1Deployment()
         deployment.api_version = api_version
         deployment.kind = kind
-        template = {"metadata": {"labels": {"app": "demo"}},
+        template = {"metadata": {"labels": labels},
                     "spec": {"containers": [
                         {"image": "quay.io/kuryr/demo",
                          "name": 'demo',
                          "ports": [{"containerPort": 8080}]}]}}
+        if failing_probe:
+            for container in template["spec"]["containers"]:
+                container["readinessProbe"] = {"httpGet": {"path": "/healthz",
+                                                           "port": 8089},
+                                               "initialDelaySeconds": 2,
+                                               "timeoutSeconds": 1}
         spec = k8s_client.V1DeploymentSpec(
             replicas=3,
             selector={"matchLabels": {"app": "demo"}}, template=template)
@@ -412,6 +458,18 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
                         namespace=namespace)
         return deployment_name, deployment_obj
 
+    def scale_deployment(self, replicas, deployment, namespace='default',
+                         label='app=demo'):
+        self.k8s_client.AppsV1Api().patch_namespaced_deployment(
+            deployment, namespace,
+            {'spec': {'replicas': replicas}})
+        # NOTE(juriarte): Wait timeout increased from 180 to 300 in order to
+        # give the pods time to transition to ready status in the gates (and
+        # slow environments).
+        self.wait_for_status(300, 15, self.check_pods_ready_num,
+                             namespace=namespace, label=label,
+                             num_pods=replicas)
+
     @classmethod
     def delete_deployment(cls, deployment_name, namespace="default"):
         api_instance = kubernetes.client.AppsV1Api()
@@ -498,7 +556,7 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
     def get_svc_ip_on_annotation(cls, service_name, namespace):
         api = cls.k8s_client.CoreV1Api()
         start = time.time()
-        while time.time() - start < consts.LB_TIMEOUT:
+        while time.time() - start < CONF.kuryr_kubernetes.lb_build_timeout:
             time.sleep(5)
             service = api.read_namespaced_service(service_name, namespace)
             if service.status.load_balancer.ingress:
@@ -533,9 +591,10 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
     def get_svc_ip_on_crd(cls, service_name, namespace):
         api = cls.k8s_client.CoreV1Api()
         start = time.time()
-        while time.time() - start < consts.LB_TIMEOUT:
+        while time.time() - start < CONF.kuryr_kubernetes.lb_build_timeout:
             time.sleep(5)
             service = api.read_namespaced_service(service_name, namespace)
+            crd_lb_ip = None
             if service.status.load_balancer.ingress:
                 try:
                     klb_crd = cls.get_kuryr_loadbalancer_crds(
@@ -787,6 +846,15 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
         endpoint = cls.k8s_client.V1Endpoints()
         endpoint.metadata = cls.k8s_client.V1ObjectMeta(name=service_name)
         # EndpointSubset is a group of addresses with a set of ports
+        try:
+            ports = [cls.k8s_client.V1EndpointPort(
+                name=port_name, port=target_port, protocol=protocol)]
+        except AttributeError:
+            # FIXME(dulek): kubernetes==21.7.0 renamed V1EndpointPort to
+            # CoreV1EndpointPort, probably mistakenly. Bugreport:
+            # https://github.com/kubernetes-client/python/issues/1661
+            ports = [cls.k8s_client.CoreV1EndpointPort(
+                name=port_name, port=target_port, protocol=protocol)]
         endpoint.subsets = [cls.k8s_client.V1EndpointSubset(
                             addresses=[
                                 cls.k8s_client.V1EndpointAddress(
@@ -795,12 +863,10 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
                                     ip=pod_2_ip
                                 )
                             ],
-                            ports=[cls.k8s_client.V1EndpointPort(
-                                  name=port_name,
-                                  port=target_port,
-                                  protocol=protocol)])]
+                            ports=ports)]
         cls.k8s_client.CoreV1Api().create_namespaced_endpoints(
             namespace=namespace, body=endpoint)
+        cls.endpoint = endpoint
         if get_ip:
             cls.service_ip = cls.get_service_ip(
                 service_name, spec_type=spec_type, namespace=namespace)
@@ -824,14 +890,16 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
         return False
 
     @classmethod
-    def create_namespace(cls, name=None, labels=None,
+    def create_namespace(cls, name=None, labels=None, annotations=None,
                          wait_for_crd=True,
                          timeout_period=consts.NS_TIMEOUT):
+        if CONF.kuryr_kubernetes.trigger_namespace_upon_pod:
+            wait_for_crd = False
         if not name:
             name = data_utils.rand_name(prefix='kuryr-namespace')
         namespace = cls.k8s_client.V1Namespace()
-        namespace.metadata = cls.k8s_client.V1ObjectMeta(name=name,
-                                                         labels=labels)
+        namespace.metadata = cls.k8s_client.V1ObjectMeta(
+            name=name, labels=labels, annotations=annotations)
         namespace_obj = cls.k8s_client.CoreV1Api().create_namespace(
             body=namespace)
 
@@ -846,9 +914,10 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
 
             return name, namespace_obj
 
-        start = time.time()
         if CONF.kuryr_kubernetes.kuryrnetworks:
+            start = time.time()
             # wait until kuryrnetwork CRD gets populated
+            kns_crd = {}
             while time.time() - start < timeout_period:
                 time.sleep(5)
                 try:
@@ -866,7 +935,7 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             kuryr_crd_annotation = K8S_ANNOTATION_PREFIX + "-net-crd"
             # wait until namespace gets created
             while True:
-                time.sleep(1)
+                time.sleep(10)
                 ns = cls.k8s_client.CoreV1Api().read_namespace_status(name)
                 if (ns.metadata.annotations and
                         ns.metadata.annotations.get(kuryr_crd_annotation)):
@@ -927,25 +996,56 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             namespace=namespace, plural=KURYR_LOAD_BALANCER_CRD_PLURAL,
             name=name)
 
-    def get_pod_list(self, namespace='default', label_selector=''):
-        return self.k8s_client.CoreV1Api().list_namespaced_pod(
+    @classmethod
+    def wait_for_kuryr_resource(cls, namespace, resource_kind, name,
+                                status_key=None):
+        start = time.time()
+        while time.time() - start < consts.KURYR_RESOURCE_CHECK_TIMEOUT:
+            time.sleep(5)
+            try:
+                resource_crd = cls.k8s_client.CustomObjectsApi(
+                    ).get_namespaced_custom_object(
+                        group=KURYR_CRD_GROUP, version=KURYR_CRD_VERSION,
+                        namespace=namespace, plural=resource_kind, name=name)
+            except kubernetes.client.rest.ApiException:
+                continue
+            if not status_key:
+                return resource_crd
+            try:
+                if resource_crd['status'][status_key]:
+                    return resource_crd
+                continue
+            except KeyError:
+                continue
+        msg = ("Timed out waiting for %(resource_kind)s/%(resource_name)s "
+               "status %(status_key)s" % {"resource_kind": resource_kind,
+                                          "resource_name": name,
+                                          "status_key": status_key})
+        raise lib_exc.TimeoutException(msg)
+
+    @classmethod
+    def get_pod_list(cls, namespace='default', label_selector=''):
+        return cls.k8s_client.CoreV1Api().list_namespaced_pod(
             namespace=namespace, label_selector=label_selector).items
 
-    def get_pod_name_list(self, namespace='default', label_selector=''):
-        return [pod.metadata.name for pod in self.get_pod_list(
+    @classmethod
+    def get_pod_name_list(cls, namespace='default', label_selector=''):
+        return [pod.metadata.name for pod in cls.get_pod_list(
             namespace=namespace, label_selector=label_selector)]
 
-    def get_pod_ip_list(self, namespace='default', label_selector=''):
-        return [pod.status.pod_ip for pod in self.get_pod_list(
+    @classmethod
+    def get_pod_ip_list(cls, namespace='default', label_selector=''):
+        return [pod.status.pod_ip for pod in cls.get_pod_list(
             namespace=namespace, label_selector=label_selector)]
 
-    def get_controller_pod_names(self):
+    @classmethod
+    def get_controller_pod_names(cls):
         controller_label = CONF.kuryr_kubernetes.controller_label
-        controller_pod_names = self.get_pod_name_list(
+        controller_pod_names = cls.get_pod_name_list(
             namespace=CONF.kuryr_kubernetes.kube_system_namespace,
             label_selector=controller_label)
-        self.assertNotEmpty(controller_pod_names, "Can't find controller pods "
-                            "with label %s" % controller_label)
+        cls.assertTrue(controller_pod_names, "Can't find controller pods "
+                       "with label %s" % controller_label)
         return controller_pod_names
 
     def _run_and_assert_fn(self, fn, repeats=10, responses_num=2):
@@ -1001,7 +1101,8 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             LOG.info("Unsupported protocol %s, returning", protocol)
             return
 
-        self._run_and_assert(req, pred)
+        self._run_and_assert(
+            req, pred, retry_repetitions=consts.REPETITIONS_PER_BACKEND*amount)
 
     def assert_backend_amount_from_pod(self, server_ip, amount, pod,
                                        server_port=None,
@@ -1092,9 +1193,10 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
         else:
             LOG.info("Unsupported protocol %s, returning", protocol)
             return
-        self._run_and_assert(req, pred)
+        self._run_and_assert(
+            req, pred, retry_repetitions=consts.REPETITIONS_PER_BACKEND*amount)
 
-    def _run_and_assert(self, fn, predicate, retry_repetitions=100):
+    def _run_and_assert(self, fn, predicate, retry_repetitions=20):
         resps = [fn() for _ in range(retry_repetitions)]
         predicate(self, resps)
 
@@ -1112,7 +1214,8 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
 
     @classmethod
     def _verify_klb_crd(cls, name, poll_interval=1, namespace='default',
-                        timeout_period=consts.LB_TIMEOUT, pod_num=None):
+                        timeout_period=CONF.kuryr_kubernetes.lb_build_timeout,
+                        pod_num=None):
         start = time.time()
         klb_crd_has_status = False
         while time.time() - start < timeout_period:
@@ -1136,10 +1239,10 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             raise lib_exc.TimeoutException(msg)
 
     @classmethod
-    def _verify_endpoints_annotation(cls, ep_name, ann_string,
-                                     poll_interval=1, namespace='default',
-                                     pod_num=None,
-                                     timeout_period=consts.LB_TIMEOUT):
+    def _verify_endpoints_annotation(
+            cls, ep_name, ann_string, poll_interval=1, namespace='default',
+            pod_num=None,
+            timeout_period=CONF.kuryr_kubernetes.lb_build_timeout):
         LOG.info("Look for %s string in ep=%s annotation ",
                  ann_string, ep_name)
 
@@ -1211,102 +1314,13 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
         while retry_attempts != 0:
             time.sleep(time_between_attempts)
             for controller_pod in self.get_controller_pod_names():
-                self.assertEqual("Running", self.get_pod_status(
+                self.assertEqual(status, self.get_pod_status(
                     controller_pod,
                     CONF.kuryr_kubernetes.kube_system_namespace),
                     'Kuryr controller is not in the %s state' % status
                 )
             retry_attempts -= 1
 
-    @classmethod
-    def create_route(cls, name, hostname, target_svc, namespace='default'):
-        route_manifest = {
-            'apiVersion': 'route.openshift.io/v1',
-            'kind': 'Route',
-            'metadata':
-                {
-                    'name': name,
-                },
-            'spec':
-                {
-                    'host': hostname,
-                    'to':
-                        {
-                            'kind': 'Service',
-                            'name': target_svc
-                        }
-                }
-        }
-
-        cls.k8s_client.CustomObjectsApi().create_namespaced_custom_object(
-            OS_ROUTES_API, OS_ROUTES_VERSION, namespace, OS_ROUTES_PLURAL,
-            route_manifest)
-
-        LOG.info("Route=%s created, wait for kuryr-annotation", name)
-        cls.wait_kuryr_annotation(
-            group=OS_ROUTES_API, version=OS_ROUTES_VERSION,
-            plural=OS_ROUTES_PLURAL,
-            annotation='openstack.org/kuryr-route-state', timeout_period=110,
-            name=name)
-        LOG.info("Found %s string in Route=%s annotation ",
-                 'openstack.org/kuryr-route-state', name)
-
-    @classmethod
-    def verify_route_endpoints_configured(cls, ep_name, namespace='default'):
-        cls._verify_endpoints_annotation(
-            ep_name=ep_name, ann_string=K8S_ANNOTATION_LBAAS_RT_STATE,
-            poll_interval=3)
-
-    @classmethod
-    def delete_route(cls, name, namespace='default'):
-        body = cls.k8s_client.V1DeleteOptions()
-        try:
-            cls.k8s_client.CustomObjectsApi().delete_namespaced_custom_object(
-                OS_ROUTES_API, OS_ROUTES_VERSION, namespace, OS_ROUTES_PLURAL,
-                name, body)
-        except kubernetes.client.rest.ApiException as e:
-            if e.status == 404:
-                return
-            raise
-
-        # FIXME(yboaron): Use other method (instead of constant delay)
-        # to verify that route was deleted
-        time.sleep(KURYR_ROUTE_DEL_VERIFY_TIMEOUT)
-
-    def verify_route_http(self, router_fip, hostname, amount,
-                          should_succeed=True):
-        LOG.info("Trying to curl the route, Router_FIP=%s, hostname=%s, "
-                 "should_succeed=%s", router_fip, hostname, should_succeed)
-
-        def req():
-            if should_succeed:
-                # FIXME(yboaron): From some reason for route use case,
-                # sometimes only one of service's pods is responding to CURL
-                # although all members and L7 policy were created at Octavia.
-                # so as workaround - I added a constant delay
-                time.sleep(1)
-
-            resp = requests.get('http://{}'.format(router_fip),
-                                headers={'Host': hostname})
-            return resp.status_code, resp.content
-
-        def pred(tester, responses):
-            contents = []
-            for resp in responses:
-                status_code, content = resp
-                if should_succeed:
-                    contents.append(content)
-                else:
-                    tester.assertEqual(requests.codes.SERVICE_UNAVAILABLE,
-                                       status_code)
-            if should_succeed:
-                unique_resps = set(contents)
-                tester.assertEqual(amount, len(unique_resps),
-                                   'Incorrect amount of unique backends. '
-                                   'Got {}'.format(unique_resps))
-
-        self._run_and_assert(req, pred)
-
     def create_and_ping_pod(self):
         name, pod = self.create_pod()
         self.addCleanup(self.delete_pod, name)
@@ -1383,7 +1397,7 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             # Wait until kuryr-controller pools are reloaded, i.e.,
             # kuryr-controller is ready
             res = test_utils.call_until_true(
-                self.get_pod_readiness, 60, 1, kuryr_pod_name,
+                self.get_container_readiness, 60, 1, kuryr_pod_name,
                 namespace=system_namespace, container_name='controller')
             self.assertTrue(res, 'Timed out waiting for '
                                  'kuryr-controller to reload pools.')
@@ -1458,6 +1472,8 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             if cleanup:
                 self.addClassResourceCleanup(self.delete_pod, pod_name,
                                              namespace=namespace)
+
+        self.wait_until_service_LB_is_active(service_name, namespace)
         self.assert_backend_amount_from_pod(
             clusterip_svc_ip,
             pod_num,
@@ -1467,6 +1483,28 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             namespace_name=namespace)
         return pod_name
 
+    def wait_until_service_LB_is_active(self, service_name,
+                                        namespace='default', poll_interval=5):
+        if CONF.kuryr_kubernetes.kuryrloadbalancers:
+            klb_crd_id = self.get_klb_crd_id(service_name, namespace)
+            start = time.time()
+            while time.time() - start < CONF.kuryr_kubernetes.lb_build_timeout:
+                try:
+                    lb_status = self.lbaas.get_loadbalancer_status(
+                        klb_crd_id)
+                except lib_exc.NotFound:
+                    break
+                else:
+                    loadbalancer = lb_status.get("loadbalancer", {})
+                    if loadbalancer.get("provisioning_status") == "ACTIVE":
+                        LOG.info("LB is ACTIVE: %s", klb_crd_id)
+                        break
+                    time.sleep(poll_interval)
+            else:
+                msg = ("Timed out waiting for loadbalancer %s to become"
+                       " ACTIVE", klb_crd_id)
+                raise lib_exc.TimeoutException(msg)
+
     def get_curl_template(self, ip_or_cidr, extra_args='', port=False):
         ipn = netaddr.IPNetwork(ip_or_cidr)
 
@@ -1487,7 +1525,7 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
                                               namespace=namespace)
         annotated_values = [value for i, value in annotation.items()]
         start = time.time()
-        while time.time() - start < consts.LB_TIMEOUT:
+        while time.time() - start < CONF.kuryr_kubernetes.lb_build_timeout:
             time.sleep(5)
             timeout_cli, timeout_mem = self.get_listener_timeout_on_crd(
                 service_name=updated_service.metadata.name,
@@ -1530,9 +1568,9 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
                                                             kargs,
                                                             kwargs))
 
-    def get_klb_crd_id(self, service_name):
+    def get_klb_crd_id(self, service_name, svc_namespace='default'):
         return self.get_kuryr_loadbalancer_crds(
-            service_name, 'default').get('status', {}).get(
+            service_name, svc_namespace).get('status', {}).get(
             'loadbalancer', {}).get('id', None)
 
     @classmethod
@@ -1541,6 +1579,7 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
         start = time.time()
         timeout = CONF.kuryr_kubernetes.lb_members_change_timeout
         while time.time() - start < timeout:
+            time.sleep(30)
             num_members = len(cls.member_client.list_members(pool_id))
             if num_members == expected_members:
                 break
@@ -1548,3 +1587,88 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             raise lib_exc.TimeoutException("Expected num of members is %s but"
                                            " actual is %s" % (expected_members,
                                                               num_members))
+
+    @classmethod
+    def get_pod_containers_restarts(cls, pod_names, namespace='default'):
+        containers = {}
+        for pod_name in pod_names:
+            containers[pod_name] = {}
+            pod = cls.k8s_client.CoreV1Api().read_namespaced_pod(pod_name,
+                                                                 namespace)
+            for container in pod.status.container_statuses:
+                containers[pod_name][container.name] = container.restart_count
+        return containers
+
+
+class BaseReconciliationScenarioTest(BaseKuryrScenarioTest):
+
+    credentials = ['admin', 'primary', ['lb_admin', 'load-balancer_admin']]
+
+    @classmethod
+    def skip_checks(cls):
+        super(BaseReconciliationScenarioTest, cls).skip_checks()
+        if not CONF.kuryr_kubernetes.service_tests_enabled:
+            raise cls.skipException("Service tests are not enabled")
+        if not CONF.kuryr_kubernetes.enable_reconciliation:
+            raise cls.skipException("Reconciliation is not enabled")
+
+    @classmethod
+    def setup_clients(cls):
+        super(BaseReconciliationScenarioTest, cls).setup_clients()
+        cls.lbaas = cls.os_roles_lb_admin.load_balancer_v2.LoadbalancerClient()
+        cls.lsnr = cls.os_roles_lb_admin.load_balancer_v2.ListenerClient()
+
+    def check_for_resource_reconciliation(self, service_name, svc_pods,
+                                          resource, resource_id,
+                                          show_resource, namespace='default'):
+        LOG.debug("Waiting for %s to be completely gone", resource)
+        start = time.time()
+        while time.time() - start < consts.LB_TIMEOUT:
+            try:
+                time.sleep(30)
+                show_resource(resource_id)
+            except lib_exc.NotFound:
+                LOG.debug("%s sucessfully deleted", resource)
+                break
+        else:
+            msg = ("Timed Out waiting for  %s to be completely"
+                   " deleted", resource_id)
+            raise lib_exc.TimeoutException(msg)
+        start = time.time()
+        timeout = consts.LB_RECONCILE_TIMEOUT + consts.LB_TIMEOUT
+        # (digitalsimboja) We need to add both timeouts to wait for the time
+        # for both rebuilding and reconciliation of the KuryrLoadBalancer CRD
+        while time.time() - start < timeout:
+            try:
+                time.sleep(60)
+                LOG.debug("Checking for %s Reconciliation", resource)
+                status = self.get_kuryr_loadbalancer_crds(service_name,
+                                                          namespace).get(
+                                                          'status', {})
+                if resource == consts.LISTENER:
+                    listeners = status.get(resource, [])
+                    if not listeners:
+                        continue
+                    new_resource_id = listeners[0].get('id')
+                else:
+                    new_resource_id = status.get(resource, {}).get('id')
+                if new_resource_id is None:
+                    continue
+                new_lb_members = status.get('members', [])
+                if (new_resource_id == resource_id or new_resource_id is None
+                        or len(svc_pods) != len(new_lb_members)):
+                    continue
+                else:
+                    self.assertNotEqual(new_resource_id, resource_id)
+                    self.assertEqual(len(svc_pods), len(new_lb_members))
+                    break
+            except kubernetes.client.rest.ApiException:
+                continue
+        else:
+            msg = ('Timed out waiting for the %s reconciliation', resource)
+            raise lib_exc.TimeoutException(msg)
+        LOG.info("%s successfully reconciled", resource)
+        # if there is a connectivity now, that means the KuryrLoadBalancer CRD
+        # resource is reconciled
+        self.check_service_internal_connectivity(service_name=service_name,
+                                                 namespace=namespace)
