@@ -49,18 +49,27 @@ class TestNamespaceScenario(base.BaseKuryrScenarioTest):
         # Check resources are created
         namespace_name, namespace = self.create_namespace()
         self.namespaces.append(namespace)
+        ns_uid = namespace.metadata.uid
 
         existing_namespaces = [ns.metadata.name
                                for ns in self.list_namespaces().items]
 
         self.assertIn(namespace_name, existing_namespaces)
 
-        subnet_name = 'ns/' + namespace_name + '-subnet'
+        pod_name, pod = self.create_pod(labels={"app": 'pod-label'},
+                                        namespace=namespace_name)
+
         kuryr_net_crd_name = 'ns-' + namespace_name
 
         seen_subnets = self.os_admin.subnets_client.list_subnets()
         seen_subnet_names = [n['name'] for n in seen_subnets['subnets']]
 
+        subnet_name = (f"{ns_uid}/{namespace_name}")
+        if subnet_name not in seen_subnet_names:
+            subnet_name = 'ns/' + namespace_name + '-subnet'
+            if subnet_name not in seen_subnet_names:
+                subnet_name = namespace_name
+
         self.assertIn(subnet_name, seen_subnet_names)
 
         subnet_id = [n['id'] for n in seen_subnets['subnets']
@@ -82,8 +91,6 @@ class TestNamespaceScenario(base.BaseKuryrScenarioTest):
             self.assertIn(kuryr_net_crd['spec']['netId'], net_id)
 
         # Check namespace pod connectivity
-        pod_name, pod = self.create_pod(labels={"app": 'pod-label'},
-                                        namespace=namespace_name)
         self.create_setup_for_service_test(namespace=namespace_name,
                                            cleanup=False)
         self.check_service_internal_connectivity(namespace=namespace_name,
@@ -108,9 +115,9 @@ class TestNamespaceScenario(base.BaseKuryrScenarioTest):
         seen_sg_ids = [sg['id'] for sg in seen_sgs]
 
         subnet_ns1_name, net_crd_ns1 = self._get_and_check_ns_resources(
-            ns1_name, existing_namespaces, seen_sg_ids)
+            ns1, existing_namespaces, seen_sg_ids)
         subnet_ns2_name, net_crd_ns2 = self._get_and_check_ns_resources(
-            ns2_name, existing_namespaces, seen_sg_ids)
+            ns2, existing_namespaces, seen_sg_ids)
         self.assertIn('default', existing_namespaces)
 
         # Create pods in different namespaces
@@ -146,14 +153,27 @@ class TestNamespaceScenario(base.BaseKuryrScenarioTest):
         self.assertIn(consts.POD_OUTPUT,
                       self.exec_command_in_pod(pod_nsdefault_name, cmd))
 
+        seen_subnets = self.os_admin.subnets_client.list_subnets()
+        if subnet_ns1_name not in seen_subnets:
+            subnet_ns1_name = f'ns/{ns1_name}-subnet'
+            if subnet_ns1_name not in seen_subnets:
+                subnet_ns1_name = ns1_name
+        if subnet_ns2_name not in seen_subnets:
+            subnet_ns2_name = f'ns/{ns2_name}-subnet'
+            if subnet_ns2_name not in seen_subnets:
+                subnet_ns2_name = ns2_name
+
         self._delete_namespace_resources(ns1_name, net_crd_ns1,
                                          subnet_ns1_name)
         self._delete_namespace_resources(ns2_name, net_crd_ns2,
                                          subnet_ns2_name)
 
-    def _get_and_check_ns_resources(self, ns_name, existing_namespaces,
+    def _get_and_check_ns_resources(self, ns, existing_namespaces,
                                     existing_sgs):
-        subnet_ns_name = 'ns/' + ns_name + '-subnet'
+        ns_name = ns.metadata.name
+        ns_uid = ns.metadata.name
+        subnet_ns_name = f'{ns_uid}/{ns_name}'
+
         net_crd_ns_name = 'ns-' + ns_name
         self.assertIn(ns_name, existing_namespaces)
 
@@ -199,9 +219,9 @@ class TestNamespaceScenario(base.BaseKuryrScenarioTest):
         seen_sg_ids = [sg['id'] for sg in seen_sgs]
 
         subnet_ns1_name, net_crd_ns1 = self._get_and_check_ns_resources(
-            ns1_name, existing_namespaces, seen_sg_ids)
+            ns1, existing_namespaces, seen_sg_ids)
         subnet_ns2_name, net_crd_ns2 = self._get_and_check_ns_resources(
-            ns2_name, existing_namespaces, seen_sg_ids)
+            ns2, existing_namespaces, seen_sg_ids)
         self.assertIn('default', existing_namespaces)
 
         pod_nsdefault_name, pod_nsdefault = self.create_pod(
@@ -242,6 +262,16 @@ class TestNamespaceScenario(base.BaseKuryrScenarioTest):
                       self.exec_command_in_pod(pod_ns1_name, cmd, ns1_name))
 
         # Check resources are deleted
+        seen_subnets = self.os_admin.subnets_client.list_subnets()
+        if subnet_ns1_name not in seen_subnets:
+            subnet_ns1_name = f'ns/{ns1_name}-subnet'
+            if subnet_ns1_name not in seen_subnets:
+                subnet_ns1_name = ns1_name
+        if subnet_ns2_name not in seen_subnets:
+            subnet_ns2_name = f'ns/{ns2_name}-subnet'
+            if subnet_ns2_name not in seen_subnets:
+                subnet_ns2_name = ns2_name
+
         self._delete_namespace_resources(ns1_name, net_crd_ns1,
                                          subnet_ns1_name)
         self._delete_namespace_resources(ns2_name, net_crd_ns2,
@@ -274,7 +304,7 @@ class TestNamespaceScenario(base.BaseKuryrScenarioTest):
 
         start = time.time()
         while time.time() - start < TIMEOUT_PERIOD:
-            time.sleep(1)
+            time.sleep(10)
             try:
                 if CONF.kuryr_kubernetes.kuryrnetworks:
                     self.get_kuryr_network_crds(namespace)
@@ -285,7 +315,7 @@ class TestNamespaceScenario(base.BaseKuryrScenarioTest):
 
         # Also wait for the namespace removal
         while time.time() - start < TIMEOUT_PERIOD:
-            time.sleep(1)
+            time.sleep(10)
             try:
                 self.get_namespace(namespace)
             except kubernetes.client.rest.ApiException:
@@ -389,10 +419,11 @@ class TestNamespaceScenario(base.BaseKuryrScenarioTest):
 
         self.delete_namespace(ns_name)
         # wait for namespace to be deleted
+        # FIXME(itzikb) Set retries to 24 when BZ#1997120 is fixed
         retries = 120
         while True:
             try:
-                time.sleep(1)
+                time.sleep(5)
                 self.k8s_client.CoreV1Api().read_namespace(ns_name)
                 retries -= 1
                 self.assertNotEqual(0, retries,
